//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of CodePrincess: A CPCtelera game made for #bitbitjam3
//  Copyright (C) 2016 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
//
//  This program is free software: you can redistribute it and/or tablatransify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include <cpctelera.h>
#include "personajes.h"
#include "juego.h"
#include "inicio.h"
#include "sprites/tiles.h"
#include "mapas/map1.h"
#include "mapas/map2.h"
#include "mapas/map3.h"
#include "mapas/map4.h"
#include "mapas/map5.h"
#include "mapas/map6.h"
#include "mapas/map7.h"
#include "mapas/map8.h"
#include "mapas/map9.h"
#include "mapas/map10.h"
#include "sprites/bicho.h"
#include "sprites/hero.h"
#include "sprites/bombilla.h"
//#include "stdio.h"
//#include "stdlib.h"


#define SI 1
#define NO 0

#define ANCHOPROTA   (20/2)
#define CENTROPROTA  5
#define ALTOPROTA    20

#define ORIGEN_MAPA_Y 68
#define ORIGEN_MAPA cpctm_screenPtr(CPCT_VMEM_START, 0, ORIGEN_MAPA_Y)


#define ANCHO  80 - ANCHOPROTA

#define ANCHO_PANTALLA 80
#define ALTO_PANTALLA 200
#define LIMITE_DERECHO ANCHO_PANTALLA - G_HERO_00_W
#define LIMITE_ABAJO ALTO_PANTALLA - G_HERO_00_H

#define SI 1
#define NO 0

//Animaciones
#define ANIM_PAUSA        4
#define ANDAR_ABAJO       4
#define ANDAR_ARRIBA      4
#define ANDAR_LADOS     4

//Mapas
#define NUM_MAPAS 10

//Enemigos 
#define NUM_ENEMIGOS 3


enum {
	M_derecha   = 0
	,  M_izquierda
	,  M_arriba
	,  M_abajo
} EMirar;

// Estados del personaje
enum {
	ST_quieto  = 0
	,  ST_previoQuieto
	,  ST_andandoAbajo
	,  ST_andandoArriba
	,  ST_andandoDerecha
	,  ST_andandoIzquierda
} EEstados;

TProta prota;
TBicho bicho;
TBala bala;
TBombilla bombilla;

u8 i, j, cntrlbala;
u8* mapa;
u8  num_mapa;
u8 distancia;
u8 xa; 
u32 m_rndState;

u8* tileHay1;
u8* tileHay2;
u8* tileHay3;
u16 c = 1000;

//Array de Mapas
u8* const mapas[NUM_MAPAS] = { g_map1, g_map2, g_map3, g_map4, g_map5, g_map6, g_map7, g_map8, g_map9, g_map10};

//Array de bichos
TBicho Bichos1[NUM_ENEMIGOS];

const TFrame g_frames[16] = {
   { M_abajo, g_hero_00 }     ,  { M_abajo, g_hero_01 }
   ,  { M_abajo, g_hero_02 }     ,  { M_abajo, g_hero_03 }
   ,  { M_arriba, g_hero_04 }    ,  { M_arriba, g_hero_05 }
   ,  { M_arriba, g_hero_06 }    ,  { M_arriba, g_hero_07 }
   ,  { M_derecha, g_hero_08 }   ,  { M_derecha, g_hero_09 }
   ,  { M_derecha, g_hero_10 }   ,  { M_derecha, g_hero_11 }
};

TFrame* const anim_abajo[ANDAR_ABAJO] = {
   &g_frames[0], &g_frames[1], &g_frames[2], &g_frames[3],
};

TFrame* const anim_arriba[ANDAR_ARRIBA] = {
   &g_frames[4], &g_frames[5], &g_frames[6], &g_frames[7]
};

TFrame* const anim_lados[ANDAR_LADOS] = {
   &g_frames[8], &g_frames[9], &g_frames[10], &g_frames[11]
};

cpctm_createTransparentMaskTable( g_tablatrans, 0x0100, M0, 0);

u8 getRand() {
   m_rndState = cpct_nextRandom_mxor_u8 (m_rndState);
   return m_rndState % 5;
}

void inicializarPJS(){
	mapa = g_map1;
	num_mapa = 0;
	cntrlbala = 1;
	m_rndState= devolverVariable();

	// inicializamos los bichos enemigos
	for(i = 0; i < NUM_ENEMIGOS; i++){
		Bichos1[i].sprite = g_bicho_0;
		Bichos1[i].vivo1=SI;
		Bichos1[i].vivo2=SI;
		Bichos1[i].vivo3=SI;
		Bichos1[i].vivo4=SI;
		Bichos1[i].vivo5=SI;
		Bichos1[i].vida= 2;
		Bichos1[i].valentia = getRand();
		Bichos1[i].semaf = 0;
	}

	//Inicializamos el prota
	prota.x = prota.px = 15; //32
	prota.y = prota.py = 160; //70
	prota.mover  = NO;
	prota.frame  = &g_frames[0];
	prota.nframe = 0;
	prota.mira   = M_abajo;
	prota.estado = ST_quieto;
	prota.prev_estado = ST_quieto;
	prota.vida=3;

	//Inicializamos la bala
	bala.existe=NO;

	//Inicializamos bombilla
	bombilla.x=bombilla.px=35;
   	bombilla.y=bombilla.py=95;
    bombilla.sprite=g_bombilla;

	dibujarMapa();
	dibujarProta();
}

u8 getVida(){	
	return prota.vida;
}

u8 finJuego(){
	if(num_mapa==9 && (prota.x>=30 && prota.x<=51) && (prota.y>=95 && prota.y<=120) )
		return 1;
	else
		return 0;
}

void dibujarPJS(){
	ejecutarEstado();

	if(prota.mover){
		seleccionarSpriteProta();
		cpct_waitVSYNC();
		redibujarProta();
		prota.mover = NO;
	}

	
	for(i = 0; i < NUM_ENEMIGOS; i++){
		if(num_mapa == 2 && Bichos1[i].vivo1 == SI){
				dibujarBicho1(i);
				moverBicho1(i);
		   		comprobarColisionProtaBicho(i);
		}
		else if(num_mapa == 3 && Bichos1[i].vivo2 == SI){
				dibujarBicho1(i);
				moverBicho1(i);
		   		comprobarColisionProtaBicho(i);
		}
		else if(num_mapa == 4 && Bichos1[i].vivo3 == SI){
				dibujarBicho1(i);
				moverBicho1(i);
		   		comprobarColisionProtaBicho(i);
		}
		else if(num_mapa == 6 && Bichos1[i].vivo4 == SI){
				dibujarBicho1(i);
				moverBicho1(i);
		   		comprobarColisionProtaBicho(i);
		}
		else if(num_mapa == 7 && Bichos1[i].vivo5 == SI){
				dibujarBicho1(i);
				moverBicho1(i);
		   		comprobarColisionProtaBicho(i);
		}
	}
	
	if(num_mapa == 9){
		dibujarBombilla();
	}

	if(bala.existe){
		distancia++;
		redibujarDisparo();
	}
	if(getVida()==3)
		cpct_drawStringM0("3UP", cpct_getScreenPtr(CPCT_VMEM_START, 0, 58),  10, c & 0x0F);
	else if(getVida()==2)
		cpct_drawStringM0 ("2UP",  cpctm_screenPtr(CPCT_VMEM_START, 0, 58),  10, c & 0x0F);
	else
		cpct_drawStringM0 ("1UP",  cpctm_screenPtr(CPCT_VMEM_START, 0, 58),  10, c & 0x0F);
}

void dibujarBombilla(){
	u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START, bombilla.x, bombilla.y);
	cpct_drawSpriteMaskedAlignedTable (bombilla.sprite,pvmem, G_BOMBILLA_W, G_BOMBILLA_H, g_tablatrans);
}

void comprobarColisionProtaBicho(u8 bicho){
	if((prota.x <= Bichos1[bicho].x && prota.x + 10 >= Bichos1[bicho].x) &&
	 (prota.y <= (Bichos1[bicho].y + 4) && (prota.y + 20 >= Bichos1[bicho].y))){
	 	prota.vida = prota.vida - 1 ;
	 	if(num_mapa == 2){
  			Bichos1[bicho].vivo1 = NO;
	 	}
	 	else if(num_mapa ==3){
  			Bichos1[bicho].vivo2 = NO;
	 	}
	 	else if(num_mapa ==4){
  			Bichos1[bicho].vivo3 = NO;
	 	}
	 	else if(num_mapa ==6){
  			Bichos1[bicho].vivo4 = NO;
	 	}
	 	else if(num_mapa ==7){
  			Bichos1[bicho].vivo5 = NO;
	 	}
  		borrarBicho(bicho);
	}		
}

//Dibujaremos los diferentes bichos que hemos inicializado
void dibujarBicho1(u8 dibujar){
	u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START, Bichos1[dibujar].x, Bichos1[dibujar].y);
	borrarBicho(dibujar);
	if(Bichos1[dibujar].sprite == g_bicho_0)
		Bichos1[dibujar].sprite = g_bicho_1;
	else
		Bichos1[dibujar].sprite = g_bicho_0;

	cpct_drawSpriteMaskedAlignedTable (Bichos1[dibujar].sprite,pvmem, G_BICHO_0_W, G_BICHO_0_H, g_tablatrans);
}

//Borramos el bicho que se le pasa por parámetro
void borrarBicho(u8 muerto){
	u8 w, h;
	if(num_mapa == 2){
		w = 2 + (Bichos1[muerto].px & 1);
		h = 2 + (Bichos1[muerto].py & 3 ? 1:0);
		cpct_etm_drawTileBox2x4 (Bichos1[muerto].px / 2, (Bichos1[muerto].py - ORIGEN_MAPA_Y) / 4, w, h, g_map1_W,  ORIGEN_MAPA, mapa);
		if(Bichos1[muerto].vivo1==NO){
			Bichos1[muerto].x=0;
			Bichos1[muerto].y=0;
		}
	}
	else if(num_mapa == 3){
		w = 2 + (Bichos1[muerto].px & 1);
		h = 2 + (Bichos1[muerto].py & 3 ? 1:0);
		cpct_etm_drawTileBox2x4 (Bichos1[muerto].px / 2, (Bichos1[muerto].py - ORIGEN_MAPA_Y) / 4, w, h, g_map1_W,  ORIGEN_MAPA, mapa);
		if(Bichos1[muerto].vivo2==NO){
			Bichos1[muerto].x=0;
			Bichos1[muerto].y=0;
		}
	}
	else if(num_mapa == 4){
		w = 2 + (Bichos1[muerto].px & 1);
		h = 2 + (Bichos1[muerto].py & 3 ? 1:0);
		cpct_etm_drawTileBox2x4 (Bichos1[muerto].px / 2, (Bichos1[muerto].py - ORIGEN_MAPA_Y) / 4, w, h, g_map1_W,  ORIGEN_MAPA, mapa);
		if(Bichos1[muerto].vivo3==NO){
			Bichos1[muerto].x=0;
			Bichos1[muerto].y=0;
		}
	}
	else if(num_mapa == 6){
		w = 2 + (Bichos1[muerto].px & 1);
		h = 2 + (Bichos1[muerto].py & 3 ? 1:0);
		cpct_etm_drawTileBox2x4 (Bichos1[muerto].px / 2, (Bichos1[muerto].py - ORIGEN_MAPA_Y) / 4, w, h, g_map1_W,  ORIGEN_MAPA, mapa);
		if(Bichos1[muerto].vivo4==NO){
			Bichos1[muerto].x=0;
			Bichos1[muerto].y=0;
		}
	}
	else if(num_mapa == 7){
		w = 2 + (Bichos1[muerto].px & 1);
		h = 2 + (Bichos1[muerto].py & 3 ? 1:0);
		cpct_etm_drawTileBox2x4 (Bichos1[muerto].px / 2, (Bichos1[muerto].py - ORIGEN_MAPA_Y) / 4, w, h, g_map1_W,  ORIGEN_MAPA, mapa);
		if(Bichos1[muerto].vivo5==NO){
			Bichos1[muerto].x=0;
			Bichos1[muerto].y=0;
		}
	}
}

void bichoHuye(u8 mover1){
	Bichos1[mover1].colisionI  = obtenerTilePtr(Bichos1[mover1].x - 1, Bichos1[mover1].y + 4); //colision izquierda
	Bichos1[mover1].colisionAr = obtenerTilePtr(Bichos1[mover1].x + 2, Bichos1[mover1].y - 1); //colision arriba
	Bichos1[mover1].colisionD  = obtenerTilePtr(Bichos1[mover1].x + 5, Bichos1[mover1].y + 4); //colision derecha
	Bichos1[mover1].colisionAb = obtenerTilePtr(Bichos1[mover1].x + 2, Bichos1[mover1].y + 9); //colision abajo

	if(prota.x >= Bichos1[mover1].x)
		Bichos1[mover1].distx = prota.x - Bichos1[mover1].x;
	else
		Bichos1[mover1].distx = Bichos1[mover1].x - prota.x;

	if(prota.y >= Bichos1[mover1].y)
		Bichos1[mover1].disty = prota.y - Bichos1[mover1].y;
	else 
		Bichos1[mover1].disty= Bichos1[mover1].y - prota.y;

	if(Bichos1[mover1].distx < 20 && Bichos1[mover1].disty <20){
		if(prota.x >= Bichos1[mover1].x && *Bichos1[mover1].colisionI == 0){ //alejarse a la izquierda
			Bichos1[mover1].px = Bichos1[mover1].x;
			Bichos1[mover1].x --;
		}
		else if(prota.x < Bichos1[mover1].x && *Bichos1[mover1].colisionD == 0){ //alejarse a derecha
			Bichos1[mover1].px = Bichos1[mover1].x;
			Bichos1[mover1].x ++;
		}
	}
	borrarBicho(mover1);
	dibujarBicho1(mover1);
}

void bichoAtaca(u8 mover1){
	if(xa == 0){
		if((prota.x + 5) < Bichos1[mover1].x){
			Bichos1[mover1].px = Bichos1[mover1].x;
			Bichos1[mover1].x --;
			xa= 1;
		}		
		else if ((prota.x + 5) == Bichos1[mover1].x){
			xa= 1;
		}
		else{
			Bichos1[mover1].px = Bichos1[mover1].x;
			Bichos1[mover1].x ++;
			xa= 1;
		}
	}
	else{
		if((prota.y + 10) < Bichos1[mover1].y){
			Bichos1[mover1].py = Bichos1[mover1].y;
			Bichos1[mover1].y --;
			xa= 0;
		}		
		else if ((prota.y + 10) == Bichos1[mover1].y){
			xa=0;
		}
		else{
			Bichos1[mover1].py = Bichos1[mover1].y;
			Bichos1[mover1].y ++;
			xa= 0;
		}
	}
	borrarBicho(mover1);
	dibujarBicho1(mover1);
}
//////////// DIBUJAR U8 EN PANTALLA //////////////////
/*
u8 str[4];
	Bichos1[mover1].colision1 = obtenerTilePtr(Bichos1[mover1].x -1, Bichos1[mover1].y + 4);
	sprintf(str, "%d", *Bichos1[mover1].colision1);
	cpct_drawStringM0(str, cpct_getScreenPtr(CPCT_VMEM_START, 10, 80),  2, 10);
*/

void bichoPatrullar(u8 mover1){
	Bichos1[mover1].colisionI = obtenerTilePtr(Bichos1[mover1].x -1, Bichos1[mover1].y + 4);
	Bichos1[mover1].colisionD = obtenerTilePtr(Bichos1[mover1].x + 5, Bichos1[mover1].y + 4);

	if(*Bichos1[mover1].colisionI == 0 && Bichos1[mover1].semaf == 0){
		Bichos1[mover1].px = Bichos1[mover1].x;
		Bichos1[mover1].x --;
	}		
	else if(*Bichos1[mover1].colisionD == 0){
		Bichos1[mover1].semaf = 1;
		Bichos1[mover1].px = Bichos1[mover1].x;
		Bichos1[mover1].x ++;
	}
	else{
		Bichos1[mover1].semaf = 0;
	}
	borrarBicho(mover1);
	dibujarBicho1(mover1);	
}

//Movemos en el eje vertical al bicho
void moverBicho1(u8 mover1){
	if(Bichos1[mover1].valentia == 0){	//HUYE
		bichoHuye(mover1);
	}
	else if(Bichos1[mover1].valentia == 1){	//PATRULLA
		bichoPatrullar(mover1);
	}
	else{									//VA A POR TI
		bichoAtaca(mover1);
	}
}

void inicializarMapa2(){
	//borrarBichos();
	i= 0;
	while(i < 3){
		Bichos1[i].muestro == SI;
		if(i == 0 && Bichos1[i].vivo1 == SI) {
			Bichos1[i].x = Bichos1[i].px = 8;
			Bichos1[i].y = Bichos1[i].py = 84;
		}
		else if(i == 1 && Bichos1[i].vivo1 == SI){
			Bichos1[i].x = Bichos1[i].px = 68;
			Bichos1[i].y = Bichos1[i].py = 84;
		}
		else if(i == 2 && Bichos1[i].vivo1 == SI){
			Bichos1[i].x = Bichos1[i].px = 40;
			Bichos1[i].y = Bichos1[i].py = 110;
		}
		if(Bichos1[i].vivo1 == SI)
			dibujarBicho1(i);
		i++;
	}
}

void inicializarMapa3(){
	//borrarBichos();
	i= 0;
	while(i < 3){
		Bichos1[i].muestro == SI;
		if(i == 0 && Bichos1[i].vivo2 == SI) {
			Bichos1[i].valentia = getRand();
			Bichos1[i].x = Bichos1[i].px = 15;
			Bichos1[i].y = Bichos1[i].py = 170;
		}
		else if(i == 1 && Bichos1[i].vivo2 == SI){
			Bichos1[i].x = Bichos1[i].px = 60;
			Bichos1[i].y = Bichos1[i].py = 125;
		}
		else if(i == 2 && Bichos1[i].vivo2 == SI){
			Bichos1[i].valentia = getRand();
			Bichos1[i].x = Bichos1[i].px = 10;
			Bichos1[i].y = Bichos1[i].py = 83;
		}
		if(Bichos1[i].vivo2 == SI)
			dibujarBicho1(i);
		i++;
	}
}

void inicializarMapa4(){
	i= 0;
	while(i < 3){
		Bichos1[i].muestro == SI;
		if(i == 0 && Bichos1[i].vivo3 == SI) {
			Bichos1[i].valentia = getRand();
			Bichos1[i].x = Bichos1[i].px = 38;
			Bichos1[i].y = Bichos1[i].py = 80;
		}
		else if(i == 1 && Bichos1[i].vivo3 == SI){
			Bichos1[i].valentia = getRand();
			Bichos1[i].x = Bichos1[i].px = 65;
			Bichos1[i].y = Bichos1[i].py = 165;
		}
		else if(i == 2 && Bichos1[i].vivo3 == SI){
			Bichos1[i].x = Bichos1[i].px = 10;
			Bichos1[i].y = Bichos1[i].py = 165;
		}
		if(Bichos1[i].vivo3 == SI)
			dibujarBicho1(i);
		i++;
	}
}

void inicializarMapa6(){
	i= 0;
	while(i < 3){
		Bichos1[i].muestro == SI;
		if(i == 0 && Bichos1[i].vivo4 == SI) {
			Bichos1[i].x = Bichos1[i].px = 20;
			Bichos1[i].y = Bichos1[i].py = 80;
		}
		else if(i == 1 && Bichos1[i].vivo4 == SI){
			Bichos1[i].valentia = getRand();
			Bichos1[i].x = Bichos1[i].px = 60;
			Bichos1[i].y = Bichos1[i].py = 100;
		}
		else if(i == 2 && Bichos1[i].vivo4 == SI){
			Bichos1[i].valentia = getRand();
			Bichos1[i].x = Bichos1[i].px = 30;
			Bichos1[i].y = Bichos1[i].py = 165;
		}
		if(Bichos1[i].vivo4 == SI)
			dibujarBicho1(i);
		i++;
	}
}

void inicializarMapa7(){
	i= 0;
	while(i < 3){
		Bichos1[i].muestro == SI;
		if(i == 0 && Bichos1[i].vivo5 == SI) {
			Bichos1[i].valentia = getRand();
			Bichos1[i].x = Bichos1[i].px = 35;
			Bichos1[i].y = Bichos1[i].py = 80;
		}
		else if(i == 1 && Bichos1[i].vivo5 == SI){
			Bichos1[i].x = Bichos1[i].px = 70;
			Bichos1[i].y = Bichos1[i].py = 165;
		}
		else if(i == 2 && Bichos1[i].vivo5 == SI){
			Bichos1[i].valentia = getRand();
			Bichos1[i].x = Bichos1[i].px = 10;
			Bichos1[i].y = Bichos1[i].py = 140;
		}
		if(Bichos1[i].vivo5 == SI)
			dibujarBicho1(i);
		i++;
	}
}

void dibujarMapa(){
	cpct_etm_drawTilemap2x4(g_map1_W , g_map1_H , ORIGEN_MAPA, mapa);
}

void avanzarMapa(){
	if(num_mapa < NUM_MAPAS -1){
		mapa = mapas[++num_mapa];

	if(num_mapa == 1){
		prota.x = prota.px = 35;
    	prota.y = prota.py = 175;
    }
    else if(num_mapa == 2){
		prota.x = prota.px = 35;
		prota.y = prota.py = 175;
		inicializarMapa2();
    }
    else if(num_mapa == 3){
    	prota.x = prota.px = 65;
    	prota.y = prota.py = 172;	
    	inicializarMapa3();
    }
    else if(num_mapa == 4){
    	prota.x = prota.px = 68;
    	prota.y = prota.py = 76;
    	inicializarMapa4();
    }
    else if(num_mapa == 5){
    	prota.x = prota.px = 35;
    	prota.y = prota.py = 175;
    }
    else if(num_mapa == 6){
    	prota.x = prota.px = 65;
    	prota.y = prota.py = 172;
    	inicializarMapa6();

    }
    else if(num_mapa == 7){
    	prota.x = prota.px = 68;
    	prota.y = prota.py = 76;
    	inicializarMapa7();

    }

    else if(num_mapa == 8){
    	prota.x = prota.px = 35;
    	prota.y = prota.py = 175;
    }
    else{
    	prota.x = prota.px = 35;
    	prota.y = prota.py = 175;
    }
    prota.mover = SI;
    dibujarMapa();
  }
}

void retrocederMapa() {
	if (num_mapa > 0) {
   		mapa = mapas[--num_mapa];

   	if(num_mapa == 0){
   		prota.x = prota.px = 35;
    	prota.y = prota.py = 70;
    }
    else if(num_mapa == 1){
   		prota.x = prota.px = 35;
   		prota.y = prota.py = 110;
    }
    else if(num_mapa == 2){
    	prota.x = prota.px = 65;
    	prota.y = prota.py = 70;
    	inicializarMapa2();
    }
    else if(num_mapa == 3){
    	prota.x = prota.px = 2;
    	prota.y = prota.py = 75;
    	inicializarMapa3();
    }
    else if(num_mapa == 4){
    	prota.x = prota.px = 34;
    	prota.y = prota.py = 70;
    	inicializarMapa4();
    }
    else if(num_mapa == 5){
    	prota.x = prota.px = 65;
    	prota.y = prota.py = 70;
    }
    else if(num_mapa == 6){
    	prota.x = prota.px = 2;
    	prota.y = prota.py = 75;
    	inicializarMapa6();
    }
    else if(num_mapa == 7){
    	prota.x = prota.px = 34;
    	prota.y = prota.py = 70;
    	inicializarMapa7();   
    }
    else{
    	prota.x = prota.px = 34;
    	prota.y = prota.py = 70;
    }
    prota.mover = SI;
    dibujarMapa();
   }
}


void girarFrame() {
   TFrame* f = prota.frame;
   if (f->mira != prota.mira) {
   		cpct_hflipSpriteM0(G_HERO_00_W, G_HERO_00_H, f->sprite);
    	f->mira = prota.mira;
	}
}

void asignarFrame(TFrame **animacion) {
	prota.frame = animacion[prota.nframe / ANIM_PAUSA];
}

void seleccionarSpriteProta() {
	switch(prota.estado) {
		case ST_quieto:{
			if(prota.prev_estado == ST_andandoAbajo)
				prota.frame = &g_frames[0];     
			else if(prota.prev_estado == ST_andandoArriba)
				prota.frame = &g_frames[4];
			else if(prota.prev_estado == ST_andandoDerecha)
				prota.frame = &g_frames[8];
			else
				prota.frame = &g_frames[8]; 
			break;
		}
		case ST_andandoIzquierda:{	asignarFrame(anim_lados);	break; }
		case ST_andandoDerecha:{	asignarFrame(anim_lados);	break; }
		case ST_andandoArriba:{	asignarFrame(anim_arriba);	break; }
		case ST_andandoAbajo:{	asignarFrame(anim_abajo);	break; }
   }
   girarFrame();
}

void dibujarProta(){
	u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START, prota.x, prota.y);
	//cpct_drawSprite (prota.sprite,pvmem, G_HERO_00_W, G_HERO_00_H );
	cpct_drawSpriteMaskedAlignedTable (prota.frame->sprite,pvmem, G_HERO_00_W, G_HERO_00_H, g_tablatrans);
}

void borrarProta(){
	u8 w = 5 + (prota.px & 1);
	u8 h = 5 + (prota.py & 3 ? 1:0);
	cpct_etm_drawTileBox2x4 (prota.px / 2, (prota.py - ORIGEN_MAPA_Y) / 4, w, h, g_map1_W,  ORIGEN_MAPA, mapa);
}

void redibujarProta(){
	borrarProta();
	prota.px = prota.x;
	prota.py = prota.y;
	dibujarProta();
}


u8* obtenerTilePtr(u8 x, u8 y) {
  return mapa + (y-ORIGEN_MAPA_Y)/4*g_map1_W + x/2;
}

u8 haySuelo(){
	tileHay1 = obtenerTilePtr(prota.x, prota.y + ALTOPROTA + 1);
	tileHay2 = obtenerTilePtr(prota.x + CENTROPROTA, prota.y + ALTOPROTA + 1);
	tileHay3 = obtenerTilePtr(prota.x + ANCHOPROTA, prota.y + ALTOPROTA + 1);

	//se puede mover 	
	
	if(*tileHay1 == 1  && *tileHay2 == 1 && *tileHay3 == 1){
		retrocederMapa();
		return 1;
	}
	else if(*tileHay1 < 4  && *tileHay2 < 4 && *tileHay3 < 4){
		return 1;
	}
	//No se puede mover
	else{
		return 0;
	}
}

u8 hayTecho() {
	tileHay1 = obtenerTilePtr(prota.x, prota.y - 1);
	tileHay2 = obtenerTilePtr(prota.x + CENTROPROTA, prota.y - 1);
	tileHay3 = obtenerTilePtr(prota.x + ANCHOPROTA, prota.y - 1);

	if(num_mapa == 1 && *tileHay1 == 19  && *tileHay2 == 19 && *tileHay3 == 19){
		avanzarMapa();
		return 1;
	}
	
	if(*tileHay1 == 2  && *tileHay2 == 2 && *tileHay3 == 2){
		avanzarMapa();
		return 1;
	}
	else if (*tileHay1 < 4  && *tileHay2 < 4 && *tileHay3 < 4 ){
		return 1;
	}
	else{
		return 0;
	}
}

u8 hayParedI() {
	tileHay1 = obtenerTilePtr(prota.x - 1, prota.y);
	tileHay2 = obtenerTilePtr(prota.x - 1, prota.y + CENTROPROTA);
	tileHay3 = obtenerTilePtr(prota.x - 1, prota.y + ALTOPROTA);

	
	if(*tileHay1 == 1  && *tileHay2 == 1 && *tileHay3 == 1){
		retrocederMapa();
		return 1;
	}
	else if(*tileHay1 == 2  && *tileHay2 == 2 && *tileHay3 == 2){
		avanzarMapa();
		return 1;
	}
	else if (*tileHay1 < 4  && *tileHay2 < 4 && *tileHay3 < 4){
		return 1;
	}
	else{
		return 0;
	}
}

u8 hayParedD() {
	tileHay1 = obtenerTilePtr(prota.x + ANCHOPROTA + 1, prota.y);
	tileHay2 = obtenerTilePtr(prota.x + ANCHOPROTA + 1, prota.y + CENTROPROTA);
	tileHay3 = obtenerTilePtr(prota.x + ANCHOPROTA + 1, prota.y + ALTOPROTA);

	
	if(*tileHay1 == 2  && *tileHay2 == 2 && *tileHay3 == 2){
		avanzarMapa();
		return 1;
	}
	else if(*tileHay1 == 1  && *tileHay2 == 1 && *tileHay3 == 1){
		retrocederMapa();
		return 1;
	}
	else if (*tileHay1 < 4  && *tileHay2 < 4 && *tileHay3 < 4){
		return 1;
	}
	else{
		return 0;
	}

}

void moverIzquierda(){
//prota.sprite = g_hero_12;
	if(hayParedI()){
		if(prota.x > 0){  
			prota.x--; 
			prota.mira= M_izquierda;
			prota.mover  = SI;
		}
	}
}

void moverDerecha(){
//prota.sprite = g_hero_08;
	if(hayParedD()){
	   if(prota.x < ANCHO){ 
		prota.x++;
		prota.mira  = M_derecha;
		prota.mover  = SI; 
		}
	}
}

void moverAbajo(){
//prota.sprite = g_hero_00;
	if(haySuelo()){
		if(prota.y < ALTO_PANTALLA){ 
		prota.y++;
		prota.mira  = M_abajo;
		prota.mover  = SI; 
	}
	}
}

void moverArriba(){
//prota.sprite = g_hero_04;
	if(hayTecho()){
		if(prota.y > 0){   
			prota.y--;
			prota.mira  = M_arriba;
			prota.mover  = SI; 
		}
	}
}

u8 compruebaTeclas(const cpct_keyID* k, u8 numk) {
	u8 i;
	//cpct_scanKeyboard_if();
	if (cpct_isAnyKeyPressed()) {
		for(i=1; i <= numk; i++, k++) {
			if (cpct_isKeyPressed(*k))
				return i;
		}
	}
	return 0;
}

void quieto_entrar() {
	prota.prev_estado = prota.estado;
	prota.estado = ST_quieto;
	prota.mover  = SI;
}

void dibujarDisparo(){
	u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START, bala.x, bala.y);
	if(bala.mira == M_derecha || bala.mira == M_izquierda)
		cpct_drawSolidBox (pvmem, cpct_px2byteM0 (3,3), 2 ,3);
	//else if(bala.mira == M_abajo || bala.mira == M_arriba)
	else
		cpct_drawSolidBox (pvmem, cpct_px2byteM0 (3,3), 1 ,4);
	}

void borrarDisparo(){
	u8 w = 1+ (bala.px & 1);
	u8 h = 1+(bala.py & 3 ? 1:0);
	cpct_etm_drawTileBox2x4 (bala.px / 2, (bala.py - ORIGEN_MAPA_Y) / 4, w, h, g_map1_W,  ORIGEN_MAPA, mapa);
}
u8 disparando(){
	u8 posx,posy;
	u8* tileColision;
	posx=posy=0;
	switch(bala.mira){
		case M_derecha:		posx=bala.x+4;	posy=bala.y;	break;
		case M_izquierda: 	posx=bala.x-1;	posy=bala.y;	break;
		case M_abajo: 		posx=bala.x;	posy=bala.y+4;	break;
		case M_arriba: 		posx=bala.x;	posy=bala.y-1;	break;
	}
	tileColision = obtenerTilePtr(posx, posy);

	for(j = 0; j < 3; j++){
		if(posx >= Bichos1[j].x && posx <= (Bichos1[j].x + 4) && posy>=Bichos1[j].y && posy <= (Bichos1[j].y + 8)){
			if(num_mapa == 2)
				Bichos1[j].vivo1=NO;
			else if(num_mapa == 3)
				Bichos1[j].vivo2=NO;
			else if(num_mapa == 4)
				Bichos1[j].vivo3=NO;
			else if(num_mapa == 6)
				Bichos1[j].vivo4=NO;
			else if(num_mapa == 7)
				Bichos1[j].vivo5=NO;

			borrarBicho(j);
			cntrlbala = 1;
			return 0;
		}
	}

	if ( *tileColision != 0 || distancia > 20){
		bala.existe=NO;
		cntrlbala = 1;
		distancia= 0;
		borrarDisparo();
		return 0;
	}
	else{
		return 1;
	}
}
void redibujarDisparo(){
	borrarDisparo();
	bala.px = bala.x;
	bala.py = bala.y;
	if(disparando()){
		switch(bala.mira){
			case M_derecha:		bala.x++;	break;
			case M_izquierda: 	bala.x--;	break;
			case M_abajo: 		bala.y++;	break;
			case M_arriba: 		bala.y--; 	break;

		}
		dibujarDisparo();
	}
}
void disparar(){
	 switch(prota.mira){
	 	case M_derecha:
			bala.x=bala.px=prota.x+ANCHOPROTA+1; 
			bala.y=bala.py=prota.y+ALTOPROTA/2;
			bala.existe=SI;
			bala.mira=M_derecha;	
			break;
		case M_izquierda:
			bala.x=bala.px=prota.x-1;
			bala.y=bala.py=prota.y+ALTOPROTA/2;
			bala.existe=SI;
			bala.mira=M_izquierda;
			break;
		case M_abajo:
			bala.x=bala.px=prota.x+ANCHOPROTA/2;
			bala.y=bala.py=prota.y+ALTOPROTA+6;
			bala.existe=SI;
			bala.mira=M_abajo;
			break;
		case M_arriba:
			bala.x=bala.px=prota.x+ANCHOPROTA/2;
			bala.y=bala.py=prota.y-6;
			bala.existe=SI;
			bala.mira=M_arriba;
			break;
}
dibujarDisparo();
}

void quieto() {
	static const cpct_keyID keys[5] = {Key_CursorUp, Key_CursorRight, Key_CursorLeft, Key_CursorDown, Key_Space };
	u8 k = compruebaTeclas(keys, 5);
	switch(k) {
		case 0: break; // Nada que hacer
		case 1: andando_entrar(M_arriba, 1); break;
		case 2: andando_entrar(M_derecha, 2);   break;
		case 3: andando_entrar(M_izquierda, 3); break;
		case 4: andando_entrar(M_abajo, 4); break;
		case 5: 
			if(cntrlbala == 1){
				disparar(); 
				cntrlbala = 0;
			}
		break;
	}
}

void andando_entrar(u8 mirar, u8 num) {
	prota.nframe = 0;
	prota.mira   = mirar;
	prota.mover  = SI;
	if(num == 1){
		prota.estado = ST_andandoArriba;
	}
	else if(num == 2){
		prota.estado = ST_andandoDerecha;
	}
	else if(num == 3){
		prota.estado = ST_andandoIzquierda;
	}
	else{
	prota.estado = ST_andandoAbajo;
	}
}

void andando_animar(u8 mirar) {
	prota.mira  = mirar;
	if(++prota.nframe == 4*ANIM_PAUSA)
		prota.nframe = 0;
}

void andando() {
	static const cpct_keyID keys[4] = {Key_CursorUp, Key_CursorRight, Key_CursorLeft, Key_CursorDown};
	u8 k = compruebaTeclas(keys, 4);
	switch(k) {
		case 0: quieto_entrar();  break;
		case 1: moverArriba();     prota.estado = ST_andandoArriba;     andando_animar(M_arriba);   break;
		case 2: moverDerecha();    prota.estado = ST_andandoDerecha;    andando_animar(M_derecha);  break;
		case 3: moverIzquierda();  prota.estado = ST_andandoIzquierda;  andando_animar(M_izquierda);break;
		case 4: moverAbajo();      prota.estado = ST_andandoAbajo;      andando_animar(M_abajo);    break;
	}
	prota.mover = SI;
}

void ejecutarEstado() {
	if(prota.estado == 0){
		quieto(); 
	}
	else{
		andando();
	}
}
